var readyStateCheckInterval = setInterval(function () {
    if (document.readyState === "complete") {
        clearInterval(readyStateCheckInterval);
        main();
    }
}, 10);

/**
 * main function loop
 */
function main() {
    /**
     * init polyfills
     */
    CSS3.init();
    /**
     *
     * @type {window.View}
     */
    var layout = new View({
        config: {
            fullscreen: true,
            tpl: 'container',
            renderTo: 'body',
            models: window.models
        },
        init: function () {
            var me = this;
            me.processData();
        },
        fullyfill: function () {

            var container = document.getElementById('slide-container'),
                width = document.querySelector('body').offsetWidth,
                height = document.querySelector('body').offsetHeight,
                padding = (width - 1320) / 2,
                style = document.getElementById('slide-style') || document.createElement('style');

            if (width <= 1920){
                style.innerHTML = "";
                return;
            }

            if(height < 1080){
                height = 1080;
            }

            style.setAttribute('id', 'slide-style');
            style.setAttribute('type', 'text/css');
            style.innerHTML = ".slide {" +
                "padding: 0 " + padding + "px;" +
                "height: " + height + "px;"
            "}";

            document.querySelector('body').appendChild(style);

        },
        /**
         * process data from json
         */
        processData: function () {
            var me = this,
                models = me.get('config').models,
                slides = models["slides"],
                mainBackgroundColor = slides.background.color,
                mainBackgroundImage = slides.background.image,
                items = slides.items[0].items;
            me.set('slides', items);
            me.set('mainBackgroundColor', mainBackgroundColor);
            me.set('mainBackgroundImage', mainBackgroundImage);
        },
        /**
         * generate all slides from data
         */
        generateSlide: function () {
            var me = this,
                models = me.get('config').models.slides.items,
                container = document.getElementById('slide-container'),
                widthT = document.querySelector('body').offsetWidth,
                width = widthT > 1920 ? widthT : 1920,
                parent = me.get('el'),
                slidesCount = 0,
                contents;
            contents = new ContentsView({
                config: {
                    id: 'contents-view',
                    fullscreen: true,
                    tpl: 'contents',
                    renderTo: '#body'
                }
            });
            contents.render();
            contents.addElement('Początek');
            for (var i = 0, len = models.length; i < len; i++) {
                switch (models[i].type) {
                    case 'slide' :
                    {
                        var slide = new SlideView({
                            config: {
                                tpl: 'slide',
                                parent: parent,
                                renderTo: '#slide-container',
                                models: models[i]
                            }
                        });
                        contents.addElement();
                        slidesCount++;
                        slide.render();
                        slide.generateContent();
                        me.get('config').views.push(slide);
                        break;
                    }
                    case 'intro' :
                    {
                        var overlay = new OverlayView({
                            config: {
                                fullscreen: true,
                                id: 'intro',
                                tpl: 'intro',
                                renderTo: '#body',
                                models: models[i]
                            }
                        });
                        overlay.render();
                        me.get('config').views.push(overlay);
                        slidesCount++;
                        break;
                    }
                    case 'outro' :
                    {
                        var outro = new OverlayView({
                            config: {
                                fullscreen: true,
                                id: 'outro',
                                tpl: 'outro',
                                renderTo: '#body',
                                models: models[i]
                            }
                        });
                        slidesCount++;
                        outro.render();
                        me.get('config').views.push(outro);
                        break;
                    }
                }
            }
            contents.addElement('Koniec');
            me.fullyfill();
            parent.querySelector('#slide-container').style.width = (slidesCount * width) + 'px';
            me.set('slidesCount', slidesCount);
        }
    });
    /**
     * render template
     */
    layout.render();
    /**
     * and generate all slides
     */
    layout.generateSlide();
    /**
     *
     * @type {window.Controller}
     */
    var ctl = new Controller({
        config: {
            name: 'Main',
            routes: {
                "slide/{\\d+}": 'changeSlide'
            },
            views: {
                layout: ViewManager.get('view-0')
            },
            current: 0,
            isAnimated: false
        },
        /**
         *
         */
        init: function () {
            var me = this;
            me.initMobile();
            me.getRouter().restore();
            me.startPage();
            me.bindElements();
            me.bindEvents();
            me.initImages();
            me.fullyfill();
        },
        initMobile: function () {
            var me = this;
            me.portraitMobileMode();
        },
        initImages: function () {
            var me = this,
                images = document.querySelectorAll('img'),
                src = [],
                counter = 0,
                loaded = function () {
                    counter--;
                    if (counter === 0) {
                        me.fireEvent('imagesLoaded');
                    }
                };
            me.set('toLoad', 1);
            for (var i = 0, l = images.length; i < l; i++) {
                if (!images[i].complete && src.indexOf(images[i].src) < 0) {
                    counter++;
                    src.push(images[i].src);
                    $(images[i]).on('load', loaded)
                }
            }
            if (counter === 0)
                me.fireEvent('imagesLoaded');
        },
        startPage: function () {
            var me = this,
                intro = ViewManager.get('intro');
            if (!window.location.hash) {
                me.changeSlide(0);
            } else {
                jQuery(intro.get('el')).css('display', 'none');
            }
        },
        bindElements: function () {
            var me = this;
            me.set('$startBtn', document.getElementById('start'));
            me.set('$loading', document.getElementById('loading'));
        },
        /**
         * bind all events
         */
        bindEvents: function () {
            var me = this,
                arrowLeft = document.getElementById('arrow-left'),
                arrowRight = document.getElementById('arrow-right'),
                start = document.getElementById('start'),
                contents = document.querySelector('.show-contents');

            arrowLeft.addEventListener('click', me.onLeftArrowClick.bind(this));
            arrowRight.addEventListener('click', me.onRightArrowClick.bind(this));
            start.addEventListener('click', me.onStartClick.bind(this));
            contents.addEventListener('click', me.onContentsClick.bind(this));
            window.addEventListener('keydown', me.onKeyDown.bind(this));
            window.addEventListener('resize', me.fullyfill.bind(this));
            window.addEventListener('resize', layout.fullyfill.bind(layout));
            window.addEventListener('resize', me.adjustContents.bind(this));
            jQuery('#body').delegate('#refresh', 'click', function () {
                $('#slide-upper-container').css('opacity', 0);
                me.get('config').current = 0;
                window.location.hash = "#slide/" + me.get('config').current;
            });
            me.addListener('imagesLoaded', me.engineReady.bind(me));

        },
        engineReady: function () {
            var me = this,
                toLoad = me.get('toLoad'),
                $l,
                $s;
            toLoad--;
            me.set('toLoad', toLoad);
            if (toLoad === 0) {
                $l = $(me.get('$loading'));
                $s = $(me.get('$startBtn'));
                $l.animate({opacity: 0}, function () {
                    $l.css('display', 'none');
                    $s.css({opacity: 0, display: 'inline'}).animate({opacity: 1});
                });
            }

        },
        onContentsClick: function () {
            var contents = ViewManager.get('contents-view');
            if (contents.get('visible')) {
                contents.hide();
            } else {
                contents.show();
            }
        },
        /**
         * on left arrow click
         * @param e
         */
        onLeftArrowClick: function (e) {
            var me = this;
            if (e)
                e.preventDefault();
            if (me.get('config').isAnimated) {
                return true;
            }
            me.get('config').current--;
            if (parseInt(me.get('config').current) < 0) {
                me.get('config').current = 0;
            }
            window.location.hash = "#slide/" + me.get('config').current;
        },
        /**
         * on right arrow click
         * @param e
         */
        onRightArrowClick: function (e) {
            var me = this, len;
            if (e)
                e.preventDefault();
            if (me.get('config').isAnimated) {
                return true;
            }
            len = me.get('config').views.layout.get('slidesCount') - 1;
            me.get('config').current++;
            if (me.get('config').current >= len) {
                me.get('config').current = len;
            }
            window.location.hash = "#slide/" + me.get('config').current;
        },
        onStartClick: function (e) {
            var me = this, $el;
            if (me.get('config').isAnimated) {
                return true;
            }
            $el = jQuery(ViewManager.get('intro').get('el'));
            me.get('config').isAnimated = true;
            $el.animate({opacity: 0}, 500, function () {
                jQuery(this).css('display', 'none');
                me.get('config').isAnimated = false;
            });
            me.get('config').current++;
            window.location.hash = "#slide/" + me.get('config').current;
        },
        onKeyDown: function (e) {
            var me = this;
            //key right
            if (e.keyCode == 39) {
                me.onRightArrowClick();
            } else
            // key left
            if (e.keyCode == 37) {
                me.onLeftArrowClick();
            }
        },
        /**
         * change slide
         * @param id
         */
        portraitMobileMode: function () {
            var me = this,
                media,
                realWidth,
                mobile;
            if (window.innerHeight > window.innerWidth) {
                if (window.matchMedia)
                    media = window.matchMedia('(max-device-width: 767px)').matches;
                mobile = isMobile();
                realWidth = (jQuery(window).width() / (window.devicePixelRatio || 1)) < 768;
                if (media || mobile && realWidth) {
                    me.fullyfillMobile();
                    me.set('portraitMobileMode', true);
                } else {
                    me.set('portraitMobileMode', false);
                    me.resetFullyfillMobile();
                }
            } else {
                me.set('portraitMobileMode', false);
                me.resetFullyfillMobile();
            }
        },
        changeSlide: function (id) {
            var me = this,
                contents = ViewManager.get('contents-view'),
                config = me.get('config'),
                layout = config.views.layout,
                widthT = document.querySelector('body').offsetWidth,
                width = widthT > 1920 ? widthT : 1920,
                container = layout.get('el').querySelectorAll('#slide-container').item(0),
                buttonRight = document.getElementById('arrow-right'),
                max = config.views.layout.get('slidesCount') - 1,
                $el;
            config.current = parseInt(id);
            contents.get('contentsElements')[id].setActive();
            // show container if not first page
            if (id > 0) {
                $('#slide-upper-container').css('opacity', 1);
                $el = jQuery(ViewManager.get('intro').get('el'));
                if ($el.css('display') === 'block') {
                    config.isAnimated = true;
                    $el.animate({opacity: 0}, 500, function () {
                        jQuery(this).css('display', 'none');
                        config.isAnimated = false;
                    });
                }
            }
            if (config.current == max) {
                buttonRight.style.display = "none";
            } else {
                buttonRight.style.display = "block";
            }
            config.isAnimated = true;
            // animate slide
            jQuery(container).animate({
                opacity: 0
            }, 500, function () {
                jQuery(this)
                    .css('transform', "translate(" + ((-width * parseInt(id - 1))) + "px, 0px)")
                    .animate({
                        opacity: 1
                    }, 500, function () {
                        config.isAnimated = false;
                    });

                // if first page
                if (config.current === 0) {
                    //show intro
                    var intro = ViewManager.get('intro');
                    if (intro.isInDOM()) {
                        jQuery(intro.get('el')).css('display', 'block');
                        jQuery(intro.get('el')).animate({opacity: 1}, 1000, function () {
                        });
                    }
                }
            });
            var outro = ViewManager.get('outro');
            // if last page
            if (config.current === max) {
                // show outro
                jQuery(outro.get('el')).css('display', 'block');
                jQuery(outro.get('el')).animate({
                    opacity: 1
                }, 500, function () {

                });
            } else {
                // else hide it
                jQuery(outro.get('el')).animate({
                    opacity: 0
                }, 500, function () {
                    jQuery(outro.get('el')).css('display', 'none');
                });
            }
            me.fullyfill();
        },
        /**
         * match content to height and width
         */
        fullyfill: function () {
            var me = this,
                len = me.get('config').views.layout.get('slidesCount') - 1;
            //me.adjustContents();
            // if last page do nothing
            if (me.get('config').current === len) {
                return;
            }
            me.portraitMobileMode();
            // if mobile portrait use other fullyfill
            if (me.get('portraitMobileMode')) {
                return;
            }
            var containerUpper = layout.get('el').querySelectorAll('#slide-upper-container').item(0),
                slides = layout.get('el').querySelectorAll('.slide'),
                ratioX = 1,
                ratioY = 1,
                ratio,
                diffX = 0,
                diffY = 0,
                offsetHeight = containerUpper.offsetHeight,
                i = me.get('config').current - 1;
            if (i < 0) {
                return;
            }
            var slide = slides.item(i);
            if (offsetHeight < slide.offsetHeight) {
                ratioY = offsetHeight / slide.offsetHeight;
                diffY = (offsetHeight - (isMobile() ? (40) : 0)) - slide.offsetHeight;
            }
            if (containerUpper.offsetWidth < slide.offsetWidth) {
                diffX = containerUpper.offsetWidth - slide.offsetWidth;
                ratioX = containerUpper.offsetWidth / slide.offsetWidth;
            }
            ratio = ratioY < ratioX ? ratioY : ratioX;
            if (ratio < 1) {
                if (diffX < 0 || diffY < 0) {
                    slide.style[CSS3.get('transform')] = "translate" + (CSS3.has3d() ? "3d(" + ((diffX / 2) ) + "px, " + ((diffY / 2)) + "px, 0)" : "(" + ((diffX / 2) ) + "px, " + ((diffY / 2)) + "px)") + " scale(" + ratio + ")";

                } else {
                    slide.style[CSS3.get('transform')] = "scale(" + ratio + ")";
                }
            } else {
                slide.style[[CSS3.get('transform')]] = "";
            }
        },
        fullyfillMobile: function () {
            var me = this;
            // run only when needed
            if (me.get('fullyfillMobile')) {
                return;
            }
            var nodes = document.querySelectorAll('.slide'),
                css = {
                    paddingLeft: 120,
                    paddingRight: (1920 - (jQuery(window).width() - 120)),
                    // you need to subtracts 1 pixel
                    // the most strange bug on android browser I ever saw
                    // HTC ONE native browser
                    width: parseInt(jQuery(window).width() - 240 - 0.001),
                    height: jQuery(window).height()
                };
            css[CSS3.get('transform')] = "";
            for (var i = 0, len = nodes.length; i < len; i++) {
                var slide = nodes.item(i);
                jQuery(slide).css(css);
            }
            me.set('fullyfillMobile', true);
        },
        resetFullyfillMobile: function () {
            var me = this,
                css;
            // run only when needed
            if (!me.get('fullyfillMobile')) {
                return;
            }
            css = {
                paddingLeft: 300,
                paddingRight: 300,
                width: 1320,
                height: 1080
            };
            var nodes = document.querySelectorAll('.slide');
            for (var i = 0, len = nodes.length; i < len; i++) {
                //another android bug simply removeAttribute('style')
                // doesnt work
                jQuery(nodes.item(i)).css(css);
            }
            me.set('fullyfillMobile', false);
        },
        adjustContents: function () {
            var contents = ViewManager.get('contents-view');
            contents.adjustTopPosition();
        }
    });
}